package com.sxtvs.open.api.review.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.ImmutableMap;
import com.sxtvs.open.api.review.dto.PageReqDTO;
import com.sxtvs.open.api.review.dto.VideoReviewResp;
import com.sxtvs.open.api.review.entity.ReviewImgRecord;
import com.sxtvs.open.api.review.entity.ReviewVideosRecord;
import com.sxtvs.open.api.review.mapper.ReviewVideosRecordMapper;
import com.sxtvs.open.api.review.service.IReviewVideosRecordService;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.auth.AESUtil;
import com.sxtvs.open.core.auth.HttpContextUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.TextUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2024-02-05
 */
@Service
@Slf4j
public class ReviewVideosRecordServiceImpl extends ServiceImpl<ReviewVideosRecordMapper, ReviewVideosRecord> implements IReviewVideosRecordService {

    @Resource
    private VcrService vcrService;

    public ReviewVideosRecord store(ReviewVideosRecord reviewVideosRecord) {
        Optional<String> token = HttpContextUtil.getToken();

        String uid = AESUtil.decryptStr(token.get());
        reviewVideosRecord.setUid(Long.valueOf(uid));
        reviewVideosRecord.setCreateTime(LocalDateTime.now());
        ResponseEntity<VideoReviewResp> resp = vcrService.reviewVideo(JSON.toJSONString(ImmutableMap.of("source", reviewVideosRecord.getUrl())));
        VideoReviewResp body = resp.getBody();

        if (null == body){
            log.error("视频审核失败 {}", resp.getStatusCode());
            this.save(reviewVideosRecord);
            throw new BizException("视频审核失败,系统正在尝试重新提交 SHARES稍后");
        }
        body.fill();
        reviewVideosRecord.setTaskId(body.getTaskId());
        reviewVideosRecord.setStatus(body.getStatus());
        reviewVideosRecord.setResult(JSON.toJSONString(body));
        reviewVideosRecord.setReviewResult(body);
        this.save(reviewVideosRecord);
        return reviewVideosRecord;
    }

    private void refreshResult(ReviewVideosRecord reviewVideosRecord) {
        ResponseEntity<VideoReviewResp> resp = vcrService.getVideoResult(reviewVideosRecord.getTaskId());
        VideoReviewResp body = resp.getBody();
        if (null == body){
            log.error("视频审核失败 {}", resp.getStatusCode());
            throw new BizException("视频审核失败,系统正在尝试重新提交 SHARES稍后");
        }
        body.fill();
        reviewVideosRecord.setStatus(body.getStatus());
        reviewVideosRecord.setResult(JSON.toJSONString(body));
        updateById(reviewVideosRecord);
    }

    public void refreshBatchResult(){
        log.info("开始刷新视频审核结果");
        lambdaQuery().in(ReviewVideosRecord::getStatus, Arrays.asList("PROVISIONING", "PREPROCESSING", "PROCESSING")).list().forEach(this::refreshResult);
    }

    public Page<ReviewVideosRecord> list(PageReqDTO pageReqDTO){
        Page<ReviewVideosRecord> page = new Page<>(pageReqDTO.getPage(), pageReqDTO.getPageSize());
        Optional<String> token = HttpContextUtil.getToken();
        String uid = AESUtil.decryptStr(token.get());
        LambdaQueryWrapper<ReviewVideosRecord> wrapper = new LambdaQueryWrapper<>();
        return baseMapper.selectPage(page, wrapper.eq(ReviewVideosRecord::getUid, Long.valueOf(uid))
                .like(!TextUtils.isEmpty(pageReqDTO.getTitle()), ReviewVideosRecord::getTitle, pageReqDTO.getTitle()));
    }
}
